clear all
close all
T      = 512;
n      = 16E3;
sigmin = 0.5;
sigmax = 1.5;
%sigtau = (sigmax-sigmin)*rand(n,1)+sigmin; % For uniform: sigmin=0, sigmax=2.
sigtau = laplace_rnd(n,1)/5+sigmax;sigtau(sigtau <= 0) = sigmax;
xmax   = 50;
p      = 1;
order  = 2;
delta  = 2*p;
pdfn   = @(t,c)exp(-abs(t)./c)./(2*c);
%pdfx   = @(t,xmax)(abs(t)<=xmax).*ones(size(t))/(2*xmax); % Uniform.
%pdfx   = @(t,xmax)(abs(t)<=2*xmax).*(1-sign(t).*t/(2*xmax))/(2*xmax); % Triangular.
pdfx   = @(t,xmax,order)makesplinepdf(t,xmax,order);
Gn     = @(nu,c,p)((1+(2*pi*nu*c).^2).^(-p));
Gx     = @(nu,xmax,order)(sinc(2*nu*xmax).^order);
Gy     = @(nu,xmax,c,p,order)(repmat(Gx(nu,xmax,order),1,length(c)).*Gn(nu,c,p));
MSE    = @(x0,x)(mean((x(:)-x0(:)).^2));
KLD    = @(x0,x,dt)(sum(x0(:).*log(x0(:)./x(:)))/dt);
nu     = [-T/2:T/2-1]'/T;
t      = [-T/2:T/2-1]';
qmf    = {MakeONFilter('Haar') MakeONFilter('Daubechies',4) MakeONFilter('Daubechies',4) MakeONFilter('Symmlet',6)};%MakeONFilter('Daubechies',max((6+delta),10));
scale  = 3;
wav    = construct_wavelets(qmf{order},T,scale,1);
wavft  = fft(wav)./T;
I      = randperm(n);I=I(1:floor(5));
randn('state',0);

noise  = sigtau.*sum(laplace_rnd(n,p),2);
x      = sum(2*xmax*rand(n,order)-xmax,2);
y      = x + noise;

Gntheo = Gn(nu,sigtau',p);
Gytheo = Gy(nu,xmax,sigtau(I)',p,order);
Gxtheo = Gx(nu,xmax,order);
pxtheo = pdfx(t,xmax,order);
wcxtheo= FWT_PO(pxtheo,scale,qmf{order});

options.sigtau = sigtau;
options.delta  = delta;
options.C      = max(pxtheo);
lambdamax  = 8/3 + 2 + 2*sqrt(16/9 + 4);
lambdalist = linspace(lambdamax/4,2*lambdamax,20);
for il=1:length(lambdalist)
	lambda = lambdalist(il);
	pdec= deconvdensityexplicit(y,T,Gntheo,options,wavft,lambda,1,1);
	psnrout(il) = MSE(pxtheo,pdec);
end
lambda = lambdalist(psnrout==min(psnrout));
pdec= deconvdensityexplicit(y,T,Gntheo,options,wavft,lambda(end),1,1);
wcpdec = FWT_PO(pdec,scale,qmf{order});

figure;
set(gcf,'Name','Densities','NumberTitle','off');
subplot(121)
[tt,h]=fasthist(y',floor(sqrt(n)));dt=diff(tt);dt=dt(1);
plot((tt+dt/2),h/sum(h)/dt,'.');axis([3*min(tt) 3*max(tt) min(h/sum(h)/dt) max(h/sum(h)/dt)]);
title('Empirical pdf_y');

subplot(122)
t = [-T/2:T/2-1]';
plot(t,pdec,'LineWidth',1.2);hold on
plot(t,pxtheo,'k');axis([3*min(tt) 3*max(tt) min(pxtheo) max(pxtheo)]);hold off
title(sprintf('x: MSE=%.2e KLD=%.2f',MSE(pxtheo,pdec),KLD(pxtheo(pxtheo & pdec),pdec(pxtheo & pdec),1/T)));
legend('Estimated pdf_x','Theoretical pdf_x','Location','NorthWest','Orientation','Vertical');
legend boxoff;

figure;
set(gcf,'Name','Characteristic functions','NumberTitle','off');
subplot(121)
plot(nu,real(Gytheo),'.');axis([-0.2 0.2 min(real(Gxtheo)) 1]);hold off
title(sprintf('y: Re(Characteristic function)'));
legend('Theoretical','Location','NorthWest','Orientation','Vertical');
legend boxoff;
% subplot(224)
% plot(nu,imag(Gytheo),'.');axis tight;hold off
% title(sprintf('Im(Characteristic function)'));

subplot(122)
plot(nu,real(fftshift(fft(pdec))),':','LineWidth',1.2);hold on
plot(nu,real(Gxtheo),'.');axis([-0.2 0.2 min(real(Gxtheo)) 1]);hold off
title(sprintf('x: Re(Characteristic function)'));
legend('Esimated','Theoretical','Location','NorthWest','Orientation','Vertical');
legend boxoff;

figure;
set(gcf,'Name','Wavelet coefficients','NumberTitle','off');
subplot(121)
PlotWaveCoeff(wcxtheo,scale,0);
title(sprintf('Coefficients of theoretical pdf_x'));

subplot(122)
PlotWaveCoeff(wcpdec,scale,0);
title(sprintf('Coefficients of estimated pdf_x'));

figure;
set(gcf,'Name','Density parameter','NumberTitle','off');
[tt,h]=fasthist(sigtau',floor(sqrt(n)));dt=diff(tt);dt=dt(1);
bar((tt+dt/2),h/sum(h)/dt);
xlabel('\sigma_v');
title(sprintf('Histogram of noise density scale parameter'));

figure
set(gcf,'Name','Threshold','NumberTitle','off');
semilogy(lambdalist,psnrout,'-',[lambdamax lambdamax],[min(psnrout) max(psnrout)],'--b');axis tight
xlabel('\kappa');ylabel('MISE');

tilefigs

%
% Part of DensityDeconvToolbox Version:100
% Written by: Jalal Fadili, GREYC CNRS ENSICAEN
%             Christophe Chesneau, LMNO University of Caen
% Created July 2011
% This material is distributed under CeCiLL licence.
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
% 


