function pdf = laplace_pdf (x)
% usage:  laplace_pdf (x)
%
% For each element of x, compute the probability density function (PDF)
% at x of the Laplace distribution.

  if (nargin ~= 1)
    disp ('laplace_pdf (x)');
  end

  [r, c] = size (x);
  s = r * c;
  x = reshape (x, 1, s);
  pdf = zeros (1, s);

  k = find (isnan (x));
  if any (k)
    pdf(k) = NaN * ones (1, length (k));
  end
  
  k = find ((x > -Inf) & (x < Inf));
  if any (k)
    pdf(k) = exp (- abs (x(k))) / 2;
  end

  pdf = reshape (pdf, r, c);


%
% Part of DensityDeconvToolbox Version:100
% Written by: Jalal Fadili, GREYC CNRS ENSICAEN
%             Christophe Chesneau, LMNO University of Caen
% Created July 2011
% This material is distributed under CeCiLL licence.
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
% 


